<?php
/*
 * @version 4.1.0
 * @package JotCache
 * @category Joomla 3.2
 * @copyright (C) 2010-2014 Vladimir Kanich
 * @license GNU General Public License version 2 or later
 */

/**
 * PURPOSE
 * cron_recache.php file is cron job starting point for JotCache RECACHE execution.
 * Here are prepared DB and logging tasks in the class CronRecache
 * as well as registration and triggering jotcache plugins which are processing
 * the `onJotcacheRecache` event.
 */

/**
 * Joomla security named constant
 */
define('_JEXEC', 1);

/***************************
 * Settings for cron job run :
 * 1. Move and rename this file cron_recache.php to the directory outside site public access.
 * 2. The constant 'JPATH_BASE' have to contain full server path to the server file directory
 *    - the root of Joomla installation
 * 3. Set constant 'JOTCACHE_ROOT_URL' which shall point to Joomla root URL
 * 4. Set cron job run conditions with following settings :
 *    'JOTCACHE_PLUGIN_NAME' string 
 *         - have to contain name of processed jotcacheplugin
 *           (in JotCache core are two plugins available - recache, crawler)
 *    $JotcachePluginParams  array
 *         - for future usage in custom jotcacheplugins
 *    $JotcachePluginStates  array
 *         - attributes for controlling the script run
 *    $JotcacheRecacheWhere
 *         - database WHERE conditions glued with 'AND' 
 *           applied on table #__jotcache
 *           available column names (displayed also in JotCache Management View) :
 *           com,view,id,ftime,title,uri,language,browser 
 *           !!USED only for plugins performing recache based on previous database content!!     
 ***************************/
define('JPATH_BASE', '/.....');
// Use ONLY for testing - when this file located in original directory :
//define('JPATH_BASE', dirname(dirname(dirname(dirname(dirname(__FILE__))))));
// echo JPATH_BASE;
define('JOTCACHE_ROOT_URL', 'http://localhost/joomla32');
define('JOTCACHE_PLUGIN_NAME', 'crawler');
//define('JOTCACHE_PLUGIN_NAME', 'recache');
$JotcachePluginParams = array();
$JotcachePluginStates = array('depth' => '2');
//$JotcacheRecacheWhere = array("view='article'","id=42");
$JotcacheRecacheWhere = array();
/***************************/

require_once JPATH_BASE . '/includes/defines.php';
require_once JPATH_BASE . '/includes/framework.php';
require_once JPATH_BASE . '/administrator/components/com_jotcache/helpers/recacherunner.php';

try {
  $tmp = php_sapi_name();
  if (php_sapi_name() != 'cli') {
    throw new Exception('cron script is not called with CLI');
  }
  $main = new RecacheRunner();
  if(!is_object($main)){
    throw new Exception('RecacheRunner object was not created');
  }
  $db = $main->getDbo();
  if(!is_object($db)){
    throw new Exception('database object was not created');
  }
  $query = $db->getQuery(true);
  $query->clear();
  $query->update($db->quoteName('#__jotcache'))
          ->set('recache=1');
  foreach ($JotcacheRecacheWhere as $jcwhere) {
    $query->where($jcwhere);
  }
  $cursor = $db->setQuery($query)->query();
  if($cursor===false){
    throw new Exception('database query on #__jotcache table not successfull');
  }
  $cacheDir = JPATH_CACHE . '/page';
  if (!file_exists($cacheDir)) {
    mkdir($cacheDir, 0755);
  }
  $flagPath = $cacheDir . '/jotcache_recache_flag_tmp.php';
  $num_bytes = file_put_contents($flagPath, "defined('_JEXEC') or die;", LOCK_EX);
  if($num_bytes===false){
    throw new Exception('cannot write into /cache/page directory');
  }
  $main->doExecute(JOTCACHE_ROOT_URL, JOTCACHE_PLUGIN_NAME, $JotcachePluginParams, $JotcachePluginStates);
  if (file_exists($flagPath)) {
    unlink($flagPath);
  }
} catch (Exception $e) {
  // An exception has been caught, echo the message.
  exit('[JotCache recache] cron script error : '.$e->getMessage());
}
